// Tiny URL Utility
//
// This utility automates creating/looking up the tinyurl for a 'long' URL.
// There is no visible or command line UI at all - the entire interface
// is managed via the clipboard.
//
// One usage model is as follows:
//
// 1) copy the (long) url to the clipboard
// 2) run tinyurl.exe
// 3) paste the (tiny) url from the clipboard
//
// What I do is help this along by putting a shortcut to tinyurl.exe
// on my desktop, and assigning the hotkey CTRL-SHIFT-T to it.
// Having done this, the above procedure looks like this:
//
// 1) ctrl-c
// 2) ctrl-shift-t
// 3) ctrl-v
//
// Enjoy.
//
// Mike Woodring
// http://www.bearcanyon.com
// http://www.pluralsight.com/blogs/mike
//
// Copyright (c) 2005-20066 Bear Canyon Consulting LLC
//
using System;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Net;
using System.Web;

namespace BearCanyon
{
	public class TinyURLForm : System.Windows.Forms.Form
	{
        private System.Windows.Forms.Label _status;
		private System.ComponentModel.Container components = null;

		public TinyURLForm()
		{
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		private void InitializeComponent()
		{
            this._status = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // _status
            // 
            this._status.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this._status.BackColor = System.Drawing.Color.Khaki;
            this._status.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this._status.Location = new System.Drawing.Point(8, 8);
            this._status.Name = "_status";
            this._status.Size = new System.Drawing.Size(232, 16);
            this._status.TabIndex = 0;
            this._status.Text = "Stand by.  Fetching tiny URL...";
            // 
            // TinyURLForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.SystemColors.Window;
            this.ClientSize = new System.Drawing.Size(248, 32);
            this.Controls.Add(this._status);
            this.ForeColor = System.Drawing.SystemColors.ControlText;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.Name = "TinyURLForm";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.ResumeLayout(false);

        }
		#endregion

		[STAThread]
		static void Main() 
		{
            // Display the status window so that users have an indication
            // of when the tiny URL is actually available in the clipboard
            // (when form goes away).
            //
            TinyURLForm f = new TinyURLForm();
            f.Show();
            f.Update();
			
            try
            {
                // Grab the long URL from the clipboard.
                //
                string longURL = ClipboardText.Get();

                if( longURL != null )
                {
                    // Get the tiny url for it.
                    //
                    string tinyURL = GetTinyURL(longURL);

                    if( tinyURL != null )
                    {
                        // If we succeeded in getting a tiny url, copy
                        // it into the clipboard.
                        //
                        ClipboardText.Put(tinyURL);
                    }
                }
            }
            catch
            {
            }
            finally
            {
                Application.Exit();
            }
        }
	
        static string GetTinyURL( string url )
        {
            string tinyurl = null;

            try
            {
                // Build & submit the POST request to tinyurl.com.
                //
                url = HttpUtility.HtmlDecode(url);
                byte[] requestData = Encoding.UTF8.GetBytes("url=" + HttpUtility.UrlEncode(url));

                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create("http://tinyurl.com/create.php");
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                request.ContentLength = requestData.Length;

#if (false)
            // These headers will make the POST packet look more exactly like the
            // POST packet sent by the tinyurl.com UI page.  But they're evidently
            // not required.  I've left them here for future reference.

            request.Referer = "http://tinyurl.com/";
            request.KeepAlive = true;
            request.Accept = "image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-shockwave-flash, */*";
            request.Headers["Cache-Control"] = "no-cache";
            request.Headers["Accept-Encoding"] = "gzip, deflate";
            request.Headers["Accept-Language"] = "en-us";
#endif

                // Without the following, the POST packet will include an "Expect"
                // header equal to "100-continue".  This causes the tinyurl.com
                // server to reply with a "417 Expectation Failed" reply when
                // GetRequestStream is called.
                //
                ServicePointManager.Expect100Continue = false;

                using( Stream requestStream = request.GetRequestStream() )
                {
                    requestStream.Write(requestData, 0, requestData.Length);
                }
        
                // Grab the response from tinyurl.com.
                //
                StreamReader response = new StreamReader(request.GetResponse().GetResponseStream());
                string responseData = response.ReadToEnd();

                // Locate the super, special, secret hidden input tag that tinyurl.com
                // includes in its response (they do this so that on >= IE4 some javascript
                // on the response page can copy the URL to the clipboard).  Once this <input> tag
                // is located, grab the value attribute.
                //
                Regex regexp = new Regex("<input +type=hidden +name=tinyurl +value=[\"]?(?<tinyurl>[^\"]+)[\"]? *>", RegexOptions.IgnoreCase);
                Match match = regexp.Match(responseData);

                if( match.Success )
                {
                    // Got it!
                    //
                    tinyurl = match.Groups["tinyurl"].Value;
                }
            }
            catch
            {
            }

            return(tinyurl);
        }
    }

    public class ClipboardText
    {
        public static string Get()
        {
            string text = null;

            IDataObject cb = Clipboard.GetDataObject();

            if( cb.GetDataPresent(DataFormats.Text) )
            {
                text = cb.GetData(DataFormats.Text) as string;
            }

            return(text);
        }

        public static void Put( string text )
        {
            Clipboard.SetDataObject(text, true);
        }
    }
}
